<?php declare(strict_types=1);

namespace App\Controller;

use App\Entity\Domain;
use App\Repository\DomainRepository;
use App\Repository\NameserverRepository;
use App\Repository\PanelRepository;
use Monolog\Logger;

error_reporting(error_level: E_ALL);

// TODO check include "/etc/bind/local.zones";

/**
 *
 */
class DomainController
{
	private string $localZoneFile;
	private string $localZonesDir;
	private string $namedConfLocalFile;
	private string $zoneCachePath;
	private string $keyhelpNamedCond;
	
	
	public function __construct(private NameserverRepository $nameserverRepository, private ApiController $checkController, private DomainRepository $domainRepository, private PanelRepository $panelRepository,  private array $config,  private Logger $log)
	{
		
		if ($this->config['debug']) {
			$this->log->debug(message: "__construct()");
		}
		
		$this->localZoneFile = '/etc/bind/local.zones';
		$this->localZonesDir = '/etc/bind/zones/';
		$this->namedConfLocalFile = '/etc/bind/named.conf.local';
		$this->zoneCachePath = '/var/cache/bind/';
		$this-> keyhelpNamedCond = '/etc/bind/named.conf.keyhelp';
	}
	
	
	
	function createIncludeFile()
	{
		if ($this->config['debug']) {
			$this->log->debug(message: "createIncludeFile()");
		}
		
		$domains = $this->domainRepository->findAll();
		
		$oFile = fopen(filename: $this->localZoneFile, mode: 'w');
		foreach ($domains as $domain) {
			fputs(stream: $oFile, data: 'include "' . $this->localZonesDir . $domain->getName() . '";' . PHP_EOL);
		}
		fclose(stream: $oFile);
		
		exec(command: '/usr/sbin/rndc reload');
	}
	
	
	function deleteOnNameservers(Domain $domain)
	{
		if ($this->config['debug']) {
			$this->log->debug(message: "deleteOnNameserver()");
		}
		
		$nameservers = $this->nameserverRepository->findAll();
		foreach ($nameservers as $nameserver) {
			$body = [
				'name' => $domain->getName()
			];
			if (!empty($nameserver->getAaaa())) {
				$this->checkController->sendCommand(requestType: 'DELETE', serverName: $nameserver->getName(), versionIP: 6, apiKey: $nameserver->getApikey(), command: 'delete', serverType: 'nameserver', body: $body);
			} else {
				$this->checkController->sendCommand(requestType: 'DELETE', serverName: $nameserver->getName(), versionIP: 4, apiKey: $nameserver->getApikey(), command: 'delete', serverType: 'nameserver', body: $body);
			}
		}
	}
	
	
	/**
	 * @param \App\Entity\Domain $domain
	 *
	 * @return void
	 */
	function deleteZone(Domain $domain)
	{
		if ($this->config['debug']) {
			$this->log->debug(message: "deleteZone()");
		}
		
		$zoneFile = $this->localZonesDir . $domain->getName();
		if (file_exists(filename: "$zoneFile")) {
			unlink(filename: $zoneFile);
		}
		$this->createIncludeFile();
		$this->deleteOnNameservers(domain: $domain);
	}
	
	
	/**
	 * @return void
	 */
	function checkPermissions(): void
	{
		if ($this->config['debug']) {
			$this->log->debug(message: "checkPermissions()");
		}
		
		echo 'Checking permission:' . PHP_EOL . PHP_EOL;
		$uid = posix_geteuid();
		print("UID:\t$uid" . PHP_EOL);
		
		$pwuid = posix_getpwuid(user_id: $uid);
		$name = $pwuid['name'];
		print("Name:\t$name" . PHP_EOL);
		$bindGroup = posix_getgrnam(name: 'bind');
		$members = $bindGroup['members'];
		if (in_array(needle: $name, haystack: $members)) {
			echo "\t✅ $name is in group 'bind" . PHP_EOL;
		} else {
			echo "\t❌$name needs to be in group $bindGroup!" . PHP_EOL;
			
		}
		
		echo 'Checking file: ' . $this->localZoneFile . PHP_EOL;
		$localZoneFilePermissions = fileperms(filename: $this->localZoneFile);
		if ($localZoneFilePermissions & 0x0010) {
			echo "\t✅ Group has write access . " . PHP_EOL;
		} else {
			echo "\t❌Group needs write permission!" . PHP_EOL;
		}
		
		echo "Checking $this->namedConfLocalFile" . PHP_EOL;
		if ($namedConfLocal = file_get_contents(filename: $this->namedConfLocalFile)) {
			if (!str_contains(haystack: $namedConfLocal, needle: $this->localZoneFile)) {
				echo "\t❌ $this->localZoneFile needs to be included in $this->namedConfLocalFile . " . PHP_EOL;
			} else {
				echo "\t✅ $this->localZoneFile is included in $this->namedConfLocalFile" . PHP_EOL;
			}
		} else {
			echo "\t❌ No access to '$this->namedConfLocalFile' . Please check permissions" . PHP_EOL;
		}
		
		echo 'Checking directory: ' . $this->localZonesDir . PHP_EOL;
		$localZoneDirPermissions = fileperms(filename: $this->localZonesDir);
		if ($localZoneDirPermissions & 0x0010) {
			echo "\t✅ Group has write access . " . PHP_EOL;
		} else {
			echo "\t❌Group needs write permission!" . PHP_EOL;
		}
	}
	
	
	/**
	 * @return array|bool
	 */
	function checkDomains(): array|bool
	{
		
		return true;
		/*
		$domains = $this->findAll();
		
		if ($namedConfLocal = file_get_contents(filename: $this->namedConfLocalFile)) {
			if (!str_contains(haystack: $namedConfLocal, needle: $this->localZoneFile)) {
				return "$this->localZoneFile needs to be included in $this->namedConfLocalFile . ";
			}
		} else {
			return "No access to '$this->namedConfLocalFile' . Please check permissions";
		}
		
		if (!fileperms($this->localZoneFile)) {
			return "No access to $this->localZoneFile . Please check permissions . ";
		}
		
		$localZones = file_get_contents($this->localZoneFile);
		
		foreach($domains as $domain) {
			if(!str_contains($localZones, $domain['name'])) {
				$errors[] = $domain['name'] . " is missing in '$this->localZoneFile'";
			}
			
			$zoneFile = $this->localZonesDir . $domain['name'];
			
			if (!file_exists($zoneFile)) {
				$errors[] = "Missing zone file for $zoneFile . Update zone to create it";
			}
		}
		
		if (empty($errors)) {
			return true;
		} else {
			return $errors;
		}
		*/
	}
	
	
	/**
	 * @param \App\Entity\Domain $domain
	 *
	 * @return void
	 */
	public function createSlaveZoneFile(Domain $domain): void
	{
		if ($this->config['debug']) {
			$domainName = $domain->getName();
			$this->log->debug(message: "createZoneFile($domainName)");
		}
		
		// check if we're a master zone
		$keyhelpConf = file_get_contents(filename: $this->keyhelpNamedCond);
		if (str_contains(haystack: $keyhelpConf, needle: $domain->getName())) {
			echo 'We a zone master for ' . $domain->getName() . PHP_EOL;
			exit(1);
		}
		
		if ($zonefile = fopen(filename: $this->localZonesDir . $domain->getName(), mode: 'w')) {
			$panelName = $domain->getPanel();
			$panel = $this->panelRepository->findByName(name: $panelName);
			$a = $panel->getA();
			$aaaa = $panel->getAaaa();
			fputs(stream: $zonefile, data: 'zone "' . $domain->getName() . '"' . ' IN {' . PHP_EOL);
			fputs(stream: $zonefile, data: "\ttype slave;" . PHP_EOL);
			fputs(stream: $zonefile, data: "\tfile \"" . $this->zoneCachePath . $domain->getName() . '.db";' . PHP_EOL);
			fputs(stream: $zonefile, data: "\tmasters {" . PHP_EOL);
			if (!empty($a)) {
				fputs(stream: $zonefile, data: "\t\t" . $a . ';' . PHP_EOL);
			}
			if (!empty($aaaa)) {
				fputs(stream: $zonefile, data: "\t\t" . $aaaa . ';' . PHP_EOL);
			}
			fputs(stream: $zonefile, data: "\t};" . PHP_EOL);
			fputs(stream: $zonefile, data: "};" . PHP_EOL);
		}
		$this->createIncludeFile();
	}
	
	
}